"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GerritApi = void 0;
const lodash_1 = __importDefault(require("lodash"));
const request_promise_1 = __importDefault(require("request-promise"));
const MAGIC_PREFIX = ')]}\'';
class GerritApi {
    constructor(inputs) {
        this.inputs = inputs;
    }
    async getBranch(repo, branch) {
        const branchInfo = await this.sendRequest(`projects/${repo}/branches/${branch}`);
        return this.sendRequest(`projects/${repo}/commits/${branchInfo.revision}`);
    }
    async getPullRequests(repo, branch) {
        return this.sendRequest(`changes/?q=branch:^${branch}+repo:${repo}&o=ALL_REVISIONS&o=ALL_COMMITS&o=WEB_LINKS&o=MESSAGES&o=CURRENT_COMMIT`);
    }
    async getCommitsInfo(repo, pr) {
        const revisions = Object.keys(pr?.revisions);
        if (revisions.length > 0) {
            const firstCommit = revisions[0];
            const lastCommit = lodash_1.default.last(revisions);
            const firstCommitDate = pr?.revisions[firstCommit].created;
            const lastCommitDate = pr?.revisions[lastCommit].created;
            const commitsByUser = {};
            const committersMap = {};
            for (const revision of revisions) {
                const commit = await this.sendRequest(`changes/${pr.id}/revisions/${revision}/commit`);
                const avatar = await this.getAvatar();
                const userName = commit.committer.name;
                committersMap[userName] = {
                    userName,
                    avatar,
                };
                if (!commitsByUser[userName]) {
                    commitsByUser[userName] = [];
                }
                commitsByUser[userName].push({
                    url: this.buildCommitUrl(repo, revision),
                    userName,
                    sha: commit.commit,
                    message: commit.subject,
                    commitDate: commit.committer.date
                });
            }
            return {
                committers: lodash_1.default.values(committersMap),
                commits: lodash_1.default.flatten(lodash_1.default.values(commitsByUser)),
                firstCommitDate,
                lastCommitDate
            };
        }
        return null;
    }
    buildCommitUrl(repo, revision) {
        return `${this.inputs.gerritHost}/plugins/gitiles/${repo}/+/${revision}`;
    }
    async getAvatar() {
        return null;
    }
    async sendRequest(query) {
        const result = await (0, request_promise_1.default)({
            method: 'GET',
            uri: `${this.inputs.gerritHost}/a/${query}`,
            auth: {
                user: this.inputs.gerritUsername,
                pass: this.inputs.gerritPassword
            },
            json: true
        });
        if (!result.startsWith(MAGIC_PREFIX)) {
            throw new Error(`magic prefix not found: ${result.slice(0, 100)}`);
        }
        try {
            return JSON.parse(result.slice(MAGIC_PREFIX.length));
        }
        catch (error) {
            throw new Error(`invalid response: ${error.message}`);
        }
    }
}
exports.GerritApi = GerritApi;
//# sourceMappingURL=gerrit.bkp.api.js.map