import _ from 'lodash'

import { RegistryClient } from '../api/registries'
import { CodefreshAPI } from '../api/codefresh'
import { ReportImageInputs, NOTIFICATIONS, NotifierFunction } from './types'

export async function reportImageInfo(inputs: ReportImageInputs, notify: NotifierFunction) {
  console.log('starting image reporter library')

  const image = inputs.imageName
  const registryClient = new RegistryClient(inputs)
  const client = await registryClient.getRegistryClient(image)

  console.log(`using ${client.constructor.name} client`)

  const workflowName = inputs.workflow?.name
  const workflowUrl = inputs.workflow?.workflowUrl
  const logsUrl = inputs.workflow?.logsUrl

  const registry = client.repoTag(image)

  const manifest = await registry.getManifest()

  console.log(`image manifest has been fetched:`, '\n', manifest)

  const config = await registry.getConfig(manifest)

  console.log(`image config has been fetched:`, '\n', config)

  const repositoryName = _.get(registryClient.parseImageName(image), 'repository')
  const imageLink = `${inputs.codefresh.host}/2.0/images/${encodeURIComponent(repositoryName)}/${manifest.config.digest}/${encodeURIComponent(image)}`

  const size = registryClient.getRegistryImageSize(manifest)


  /***
   * Notification
   ***/
  try {
    await notify(NOTIFICATIONS.IMAGE_NAME, image)
  } catch (error) {
    console.error(`Can't call notification "${NOTIFICATIONS.IMAGE_NAME}". ${error.message}`)
  }
  try {
    await notify(NOTIFICATIONS.IMAGE_SHA, _.get(manifest, 'config.digest'))
  } catch (error) {
    console.error(`Can't call notification "${NOTIFICATIONS.IMAGE_SHA}". ${error.message}`)
  }
  try {
    await notify(NOTIFICATIONS.IMAGE_LINK, imageLink)
  } catch (error) {
    console.error(`Can't call notification "${NOTIFICATIONS.IMAGE_LINK}". ${error.message}`)
  }


  const { host, apiKey } = inputs.codefresh
  const codefreshAPI = new CodefreshAPI(host, apiKey)
  const configCreated = config.created || new Date().toISOString()

  /***
   * Image Binary
  ***/
  const imageBinaryVars = {
    imageBinary: {
      id: manifest.config.digest,
      created: configCreated,
      imageName: image,
      dockerFile: await registryClient.buildDockerfile(),
      size: size,
      os: config.os,
      architecture: config.architecture,
      workflowName: workflowName,
      workflowUrl,
      logsUrl,
    }
  }

  console.log('binaryMutation payload:', imageBinaryVars.imageBinary)

  const binaryResult = await codefreshAPI.createImageBinary(imageBinaryVars)

  console.log(JSON.stringify(binaryResult, null, 2))

  /***
   * Image Layers
   ***/
  const imageLayersVars = {
    imageLayers: {
      image: manifest.config.digest,
      layers: registryClient.getLayersFromHistory(config.history, manifest.layers),
    }
  }

  console.log('ImageLayers Mutation payload:', JSON.stringify(imageLayersVars.imageLayers, null, 2))

  const imageLayersResult = await codefreshAPI.createImageLayers(imageLayersVars)

  console.log(JSON.stringify(imageLayersResult, null, 2))

  /***
   * Image Registry
   ***/
  const imageRegistryVars = {
    imageRegistry: {
      binaryId: binaryResult.createImageBinary.id,
      imageName: image,
      repoDigest: manifest.config.repoDigest,
      created: configCreated
    }
  }

  console.log('registryMutation payload:', imageRegistryVars.imageRegistry)

  const registryResult = await codefreshAPI.createImageRegistry(imageRegistryVars)

  console.log(JSON.stringify(registryResult, null, 2))

  console.log('image reported successfully')
}
